{- | this module is derived from the github source at

 "https://github.com/sdiehl/write-you-a-haskell/blob/master/chapter3/parsec.hs"

 which is a part of the excellent tutorial
 Write You a Haskell ("http://dev.stephendiehl.com/fun/")
 authored by Stephen Diehl
-}

-- ----------------------------------------

module Test.NanoParsec where

import Data.Char           (isSpace)
import Control.Monad       (when)
import Control.Applicative ((<$>), (<|>))

import Text.NanoParsec

-- ----------------------------------------

{- The grammar

 -- expr, term, factor and prim are the nonterminals
 -- of the CFG

 expr    = term { addop term }
 term    = factor { mulop factor }
 factor  = unop factor | prim
 prim    = "(" expr ")" | real

 -- the scanner:
 -- real, addop, mulop, unop, ")" and "("
 -- are the tokens

 real    = sign number [ fract ] [ expo ]
 number  = digit { digit }
 fract   = "." number
 expo    = ( "e" | "E" ) pm number
 pm      = [ "+" | "-" ]
 sign    = [ "-" ]
 digit   = "0" | "1" | ... | "8" | "9"

 addop   = "+" | "-"
 mulop   = "*" | "/"
 unop    = "sqrt" | "sin" | "cos"
-}

-- ----------------------------------------
-- the abstract syntax

data Expr
  = Add  Expr Expr
  | Sub  Expr Expr
  | Mul  Expr Expr
  | Div  Expr Expr
  | Sqrt Expr
  | Sin  Expr
  | Cos  Expr
  | Lit  Double
  deriving Show

-- ----------------------------------------
-- the mini evaluator

eval :: Expr -> Double
eval ex = case ex of
  Add  x y -> eval x + eval y
  Sub  x y -> eval x - eval y
  Mul  x y -> eval x * eval y
  Div  x y -> eval x / eval y
  Sqrt x   -> sqrt (eval x)
  Sin  x   -> sin  (eval x)
  Cos  x   -> cos  (eval x)
  Lit  x   -> x

-- ----------------------------------------
-- the expr parser combinators

real :: Parser Expr
real = undefined

expr :: Parser Expr
expr = undefined

term :: Parser Expr
term = undefined

factor :: Parser Expr
factor = undefined

addop :: Parser (Expr -> Expr -> Expr)
addop = undefined

mulop :: Parser (Expr -> Expr -> Expr)
mulop = undefined

-- --------------------
--
-- token parsers

infixOp :: String -> (a -> a -> a) -> Parser (a -> a -> a)
infixOp = undefined

unop :: String -> (a -> a) -> Parser (a -> a)
unop = undefined

leftPar :: Parser ()
leftPar  = undefined

rightPar :: Parser ()
rightPar = undefined

realLit :: Parser String
realLit = undefined

-- ----------------------------------------
-- the main prog

parse :: String -> Either String Expr
parse = runParser $ spaces >> expr

main :: IO ()
main = do
  putStr "calc> "
  ex <- getLine
  if ex == "."
    then return ()
    else do when (not (all isSpace ex)) $
              case parse ex of
              Left err -> putStrLn err
              Right e  -> print (eval e)
            main

-- ----------------------------------------
